@@ -12,7 +12,7 @@
 #include "./tiffdec.h"
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#include "webp/config.h"
 #endif
 
 #include <stdio.h>
@@ -97,7 +97,7 @@ int ReadTIFF(const char* const filename,
       pic->width = width;
       pic->height = height;
       // TIFF data is ABGR
-#ifdef __BIG_ENDIAN__
+#ifdef WORDS_BIGENDIAN
       TIFFSwabArrayOfLong(raster, width * height);
 #endif
       pic->use_argb = 1;
